==> voice-android-6.1.0-sources.jar: com/twilio/voice/CallInviteProxy.java <==
package com.twilio.voice;

import android.content.Context;
import android.content.IntentFilter;
import android.net.ConnectivityManager;
import android.os.Handler;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import android.util.Pair;

import java.util.Map;

class CallInviteProxy implements MessageListener {
    private static final Logger logger = Logger.getLogger(CallInviteProxy.class);

    private final ThreadUtils.ThreadChecker threadChecker;
    private final Context context;
    long nativeCallInviteProxy;
    private final Handler handler;
    private final MessageListener messageListener;
    private final MediaFactory mediaFactory;
    @Nullable private Call.EventListener eventListener;
    private boolean released = false;
    private ConnectivityReceiver connectivityReceiver = null;
    private Call call;
    private CallInvite callInvite;
    private String selectedRegion = Voice.region;
    private String gateway;
    private String region;
    String codecParams;
    String selectedCodec;
    private EventPublisher publisher;

    /*
     * This event listener is used at the JNI level to proxy event listener callbacks to event
     * listeners provided when accept and reject are called.
     */
    @SuppressWarnings("unused")
    private final Call.EventListener eventListenerProxy = new Call.EventListener() {
        @Override
        public void onEvent(Map<String, Pair<String, Class>> data) {
            handler.post(new Runnable() {
                @Override
                public void run() {
                    CallInviteProxy.this.threadChecker.checkIsOnValidThread();
                    logger.d("onEvent");
                    if (data.get(EventKeys.EVENT_GROUP).first.equals(EventGroupType.REGISTRATION_EVENT_GROUP)) {
                        if (data.get(EventKeys.EVENT_NAME).first.equals(EventType.UNSUPPORTED_CANCEL_MESSAGE_ERROR_EVENT)) {
                            CallInviteProxy.this.release(data.get(EventKeys.CALL_SID_KEY).first);
                        }
                    }
                    if (eventListener != null) {
                        eventListener.onEvent(data);
                    }
                    publishEvent(data);
                }
            });
        }

        @Override
        public void onMetric(Map<String, Pair<String, Class>> data) {
            handler.post(new Runnable() {
                @Override
                public void run() {
                    CallInviteProxy.this.threadChecker.checkIsOnValidThread();
                    logger.d("onMetric");
                    if (eventListener != null) {
                        eventListener.onMetric(data);
                    }

                    if (data.get(EventKeys.EVENT_GROUP).first.equals(EventGroupType.CALL_QUALITY_STATS_GROUP)) {
                        call.onSample(InsightsUtils.createRtcSample(data));
                    }
                }
            });
        }
    };

    @Override
    public void onCallInvite(final @NonNull CallInvite callInvite) {
        handler.post(new Runnable() {
            @Override
            public void run() {
                threadChecker.checkIsOnValidThread();
                CallInviteProxy.this.threadChecker.checkIsOnValidThread();
                logger.d("onCallInvite");
                Voice.callInviteProxyMap.put(callInvite.getCallSid(), CallInviteProxy.this);
                registerConnectivityBroadcastReceiver();
                messageListener.onCallInvite(callInvite);
            }
        });
    }

    @Override
    public void onCancelledCallInvite(@NonNull CancelledCallInvite cancelledCallInvite,
                                      @Nullable CallException callException) {
        handler.post(new Runnable() {
            @Override
            public void run() {
                threadChecker.checkIsOnValidThread();
                logger.d("onCancelledCallInvite: CallException code: " +
                        ((callException == null) ? "null" : callException.getErrorCode()));
                boolean wasReleased = released;
                release(cancelledCallInvite.getCallSid());

                /*
                 * Surface the callback if the call invite proxy was not released as part of calling
                 * accept or reject.
                 *
                 */
                if (!wasReleased) {
                    messageListener.onCancelledCallInvite(cancelledCallInvite, callException);
                }
            }
        });
    }

    synchronized void setEventListener(Call.EventListener eventListener) {
        this.threadChecker.checkIsOnValidThread();
        this.eventListener = eventListener;
    }

    synchronized void setCall(Call call) {
        this.threadChecker.checkIsOnValidThread();
        this.call = call;
    }

    void networkChange(Voice.NetworkChangeEvent networkChangeEvent) {
        threadChecker.checkIsOnValidThread();

        if (!released) {
            nativeNetworkChange(nativeCallInviteProxy, networkChangeEvent);
        } else {
            logger.d("Ignoring networkChangeEvent: " + networkChangeEvent.name() +
                    " because CallInviteProxy is released");
        }
    }

    synchronized void release(String callSid) {
        this.threadChecker.checkIsOnValidThread();
        logger.d("release");

        if (!released) {
            unregisterConnectivityBroadcastReceiver();

            Voice.callInviteProxyMap.remove(callSid);

            if (nativeCallInviteProxy != 0) {
                nativeRelease(nativeCallInviteProxy);
                nativeCallInviteProxy = 0;
            }

            mediaFactory.release(this);

            released = true;
        }
    }

    private CallInviteProxy(Context context,
                            long nativeCallInviteProxy,
                            Handler handler,
                            MessageListener messageListener,
                            CallInvite callInvite) {
        Preconditions.checkApplicationContext(context, "must create Call with application context");
        this.context = context;
        this.nativeCallInviteProxy = nativeCallInviteProxy;
        this.handler = handler;
        this.threadChecker = new ThreadUtils.ThreadChecker(handler.getLooper().getThread());
        this.messageListener = messageListener;
        this.mediaFactory = MediaFactory.instance(this, context);
        this.callInvite = callInvite;
        String bridgeToken = null;

        if (Voice.callSidBridgeTokenPair != null) {
            bridgeToken = Voice.callSidBridgeTokenPair.first.equals(callInvite.getCallSid()) ?
                    Voice.callSidBridgeTokenPair.second : null;
        }

        if (bridgeToken != null) {
            publisher = new EventPublisher(context, Constants.CLIENT_SDK_PRODUCT_NAME, bridgeToken);
            publisher.addListener(new EventPublisher.EventPublisherListener() {
                @Override
                public void onError(VoiceException voiceException) {
                    logger.e("Error publishing data : " + voiceException.getMessage() + ":" + voiceException.getErrorCode());
                }
            });
        }
    }

    private void registerConnectivityBroadcastReceiver() {
        connectivityReceiver = new ConnectivityReceiver();
        context.registerReceiver(connectivityReceiver, new IntentFilter(ConnectivityManager.CONNECTIVITY_ACTION));
    }

    private void unregisterConnectivityBroadcastReceiver() {
        if (connectivityReceiver != null) {
            context.unregisterReceiver(connectivityReceiver);
        }
    }

    private void publishEvent(Map<String, Pair<String, Class>> data) {
        if (data.get(EventKeys.EVENT_GROUP).first.equals(EventGroupType.SETTINGS_GROUP)) {
            if (data.get(EventKeys.EVENT_NAME).first.equals(EventType.CODEC_EVENT)) {
                codecParams = data.get(EventKeys.CODEC_PARAMS).first;
                selectedCodec = data.get(EventKeys.SELECTED_CODEC).first;
            } else if (data.get(EventKeys.EVENT_NAME).first.equals(EventType.EDGE_EVENT)) {
                gateway = (data.get(EventKeys.EDGE_HOST_NAME)).first;
                region = (data.get(EventKeys.EDGE_HOST_REGION)).first;
            }
            InsightsUtils.processEvent(data, createEventPayloadBuilderForSettingsEvent(), publisher, Constants.Direction.INCOMING);
        } else {
            InsightsUtils.processEvent(data, createEventPayloadBuilder(), publisher, Constants.Direction.INCOMING);
        }
    }

    private EventPayload.Builder createEventPayloadBuilder() {
        return new EventPayload.Builder()
                .callSid(callInvite.getCallSid())
                .messageSid(callInvite.getMessageSid())
                .direction(Constants.Direction.INCOMING)
                .selectedRegion(selectedRegion)
                .gateway(gateway)
                .region(region)
                .productName(com.twilio.voice.Constants.CLIENT_SDK_PRODUCT_NAME)
                .clientName(Utils.parseClientIdentity(callInvite.getTo()))
                .payLoadType(com.twilio.voice.Constants.APP_JSON_PAYLOADTYPE);
    }

    EventPayload.Builder createEventPayloadBuilderForSettingsEvent() {
        return createEventPayloadBuilder()
                .codecParams(codecParams)
                .selectedCodec(selectedCodec);
    }

    private native void nativeNetworkChange(long nativeCallInviteProxy,
                                            Voice.NetworkChangeEvent networkChangeEvent);
    private native void nativeRelease(long nativeCallInviteProxy);
}

==> voice-android-6.1.3-sources.jar: com/twilio/voice/CallInviteProxy.java <==
package com.twilio.voice;

import android.content.Context;
import android.content.IntentFilter;
import android.net.ConnectivityManager;
import android.os.Handler;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import android.util.Pair;

import java.util.Map;

class CallInviteProxy implements MessageListener {
    private static final Logger logger = Logger.getLogger(CallInviteProxy.class);

    private final ThreadUtils.ThreadChecker threadChecker;
    private final Context context;
    long nativeCallInviteProxy;
    private final Handler handler;
    private final MessageListener messageListener;
    private final MediaFactory mediaFactory;
    @Nullable private Call.EventListener eventListener;
    private boolean released = false;
    private ConnectivityReceiver connectivityReceiver = null;
    private Call call;
    private CallInvite callInvite;
    private String selectedRegion = Voice.region;
    private String gateway;
    private String region;
    String codecParams;
    String selectedCodec;
    private EventPublisher publisher;

    /*
     * This event listener is used at the JNI level to proxy event listener callbacks to event
     * listeners provided when accept and reject are called.
     */
    @SuppressWarnings("unused")
    private final Call.EventListener eventListenerProxy = new Call.EventListener() {
        @Override
        public void onEvent(Map<String, Pair<String, Class>> data) {
            handler.post(new Runnable() {
                @Override
                public void run() {
                    CallInviteProxy.this.threadChecker.checkIsOnValidThread();
                    logger.d("onEvent");
                    if (data.get(EventKeys.EVENT_GROUP).first.equals(EventGroupType.REGISTRATION_EVENT_GROUP)) {
                        if (data.get(EventKeys.EVENT_NAME).first.equals(EventType.UNSUPPORTED_CANCEL_MESSAGE_ERROR_EVENT)) {
                            CallInviteProxy.this.release(data.get(EventKeys.CALL_SID_KEY).first);
                        }
                    }
                    if (eventListener != null) {
                        eventListener.onEvent(data);
                    }
                    publishEvent(data);
                }
            });
        }

        @Override
        public void onMetric(Map<String, Pair<String, Class>> data) {
            handler.post(new Runnable() {
                @Override
                public void run() {
                    CallInviteProxy.this.threadChecker.checkIsOnValidThread();
                    logger.d("onMetric");
                    if (eventListener != null) {
                        eventListener.onMetric(data);
                    }

                    if (data.get(EventKeys.EVENT_GROUP).first.equals(EventGroupType.CALL_QUALITY_STATS_GROUP)) {
                        call.onSample(InsightsUtils.createRtcSample(data));
                    }
                }
            });
        }
    };

    @Override
    public void onCallInvite(final @NonNull CallInvite callInvite) {
        handler.post(new Runnable() {
            @Override
            public void run() {
                threadChecker.checkIsOnValidThread();
                CallInviteProxy.this.threadChecker.checkIsOnValidThread();
                logger.d("onCallInvite");
                Voice.callInviteProxyMap.put(callInvite.getCallSid(), CallInviteProxy.this);
                registerConnectivityBroadcastReceiver();
                messageListener.onCallInvite(callInvite);
            }
        });
    }

    @Override
    public void onCancelledCallInvite(@NonNull CancelledCallInvite cancelledCallInvite,
                                      @Nullable CallException callException) {
        handler.post(new Runnable() {
            @Override
            public void run() {
                threadChecker.checkIsOnValidThread();
                logger.d("onCancelledCallInvite: CallException code: " +
                        ((callException == null) ? "null" : callException.getErrorCode()));
                boolean wasReleased = released;
                release(cancelledCallInvite.getCallSid());

                /*
                 * Surface the callback if the call invite proxy was not released as part of calling
                 * accept or reject.
                 *
                 */
                if (!wasReleased) {
                    messageListener.onCancelledCallInvite(cancelledCallInvite, callException);
                }
            }
        });
    }

    synchronized void setEventListener(Call.EventListener eventListener) {
        this.threadChecker.checkIsOnValidThread();
        this.eventListener = eventListener;
    }

    synchronized void setCall(Call call) {
        this.threadChecker.checkIsOnValidThread();
        this.call = call;
    }

    void networkChange(Voice.NetworkChangeEvent networkChangeEvent) {
        threadChecker.checkIsOnValidThread();

        if (!released) {
            nativeNetworkChange(nativeCallInviteProxy, networkChangeEvent);
        } else {
            logger.d("Ignoring networkChangeEvent: " + networkChangeEvent.name() +
                    " because CallInviteProxy is released");
        }
    }

    synchronized void release(String callSid) {
        this.threadChecker.checkIsOnValidThread();
        logger.d("release");

        if (!released) {
            unregisterConnectivityBroadcastReceiver();

            Voice.callInviteProxyMap.remove(callSid);

            if (nativeCallInviteProxy != 0) {
                nativeRelease(nativeCallInviteProxy);
                nativeCallInviteProxy = 0;
            }

            mediaFactory.release(this);

            released = true;
        }
    }

    private CallInviteProxy(Context context,
                            long nativeCallInviteProxy,
                            Handler handler,
                            MessageListener messageListener,
                            CallInvite callInvite) {
        Preconditions.checkApplicationContext(context, "must create Call with application context");
        this.context = context;
        this.nativeCallInviteProxy = nativeCallInviteProxy;
        this.handler = handler;
        this.threadChecker = new ThreadUtils.ThreadChecker(handler.getLooper().getThread());
        this.messageListener = messageListener;
        this.mediaFactory = MediaFactory.instance(this, context);
        this.callInvite = callInvite;
        String bridgeToken = null;

        if (Voice.callSidBridgeTokenPair != null) {
            bridgeToken = Voice.callSidBridgeTokenPair.first.equals(callInvite.getCallSid()) ?
                    Voice.callSidBridgeTokenPair.second : null;
        }

        if (bridgeToken != null) {
            publisher = new EventPublisher(context, Constants.getClientSdkProductName(), bridgeToken);
            publisher.addListener(new EventPublisher.EventPublisherListener() {
                @Override
                public void onError(VoiceException voiceException) {
                    logger.e("Error publishing data : " + voiceException.getMessage() + ":" + voiceException.getErrorCode());
                }
            });
        }
    }

    private void registerConnectivityBroadcastReceiver() {
        connectivityReceiver = new ConnectivityReceiver();
        context.registerReceiver(connectivityReceiver, new IntentFilter(ConnectivityManager.CONNECTIVITY_ACTION));
    }

    private void unregisterConnectivityBroadcastReceiver() {
        if (connectivityReceiver != null) {
            context.unregisterReceiver(connectivityReceiver);
        }
    }

    private void publishEvent(Map<String, Pair<String, Class>> data) {
        if (data.get(EventKeys.EVENT_GROUP).first.equals(EventGroupType.SETTINGS_GROUP)) {
            if (data.get(EventKeys.EVENT_NAME).first.equals(EventType.CODEC_EVENT)) {
                codecParams = data.get(EventKeys.CODEC_PARAMS).first;
                selectedCodec = data.get(EventKeys.SELECTED_CODEC).first;
            } else if (data.get(EventKeys.EVENT_NAME).first.equals(EventType.EDGE_EVENT)) {
                gateway = (data.get(EventKeys.EDGE_HOST_NAME)).first;
                region = (data.get(EventKeys.EDGE_HOST_REGION)).first;
            }
            InsightsUtils.processEvent(data, createEventPayloadBuilderForSettingsEvent(), publisher, Constants.Direction.INCOMING);
        } else {
            InsightsUtils.processEvent(data, createEventPayloadBuilder(), publisher, Constants.Direction.INCOMING);
        }
    }

    private EventPayload.Builder createEventPayloadBuilder() {
        return new EventPayload.Builder()
                .callSid(callInvite.getCallSid())
                .messageSid(callInvite.getMessageSid())
                .direction(Constants.Direction.INCOMING)
                .selectedRegion(selectedRegion)
                .gateway(gateway)
                .region(region)
                .productName(com.twilio.voice.Constants.getClientSdkProductName())
                .clientName(Utils.parseClientIdentity(callInvite.getTo()))
                .payLoadType(com.twilio.voice.Constants.APP_JSON_PAYLOAD_TYPE);
    }

    EventPayload.Builder createEventPayloadBuilderForSettingsEvent() {
        return createEventPayloadBuilder()
                .codecParams(codecParams)
                .selectedCodec(selectedCodec);
    }

    private native void nativeNetworkChange(long nativeCallInviteProxy,
                                            Voice.NetworkChangeEvent networkChangeEvent);
    private native void nativeRelease(long nativeCallInviteProxy);
}
